// Copyright (C) 2022 The Qt Company
// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only

#include "qtmultimediaquicktypes_p.h"

QT_BEGIN_NAMESPACE

// TODO...

QT_END_NAMESPACE

#include "moc_qtmultimediaquicktypes_p.cpp"
