/****************************************************************************
** Meta object code from reading C++ file 'qffmpegrenderer_p.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../../qtmultimedia/src/plugins/multimedia/ffmpeg/playbackengine/qffmpegrenderer_p.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qffmpegrenderer_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN7QFFmpeg8RendererE_t {};
} // unnamed namespace

template <> constexpr inline auto QFFmpeg::Renderer::qt_create_metaobjectdata<qt_meta_tag_ZN7QFFmpeg8RendererE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QFFmpeg::Renderer",
        "frameProcessed",
        "",
        "Frame",
        "synchronized",
        "PlaybackEngineObjectID",
        "id",
        "TimePoint",
        "tp",
        "TrackPosition",
        "pos",
        "forceStepDone",
        "loopChanged",
        "offset",
        "index",
        "onFinalFrameReceived",
        "sourceID",
        "render"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'frameProcessed'
        QtMocHelpers::SignalData<void(Frame)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 },
        }}),
        // Signal 'synchronized'
        QtMocHelpers::SignalData<void(PlaybackEngineObjectID, TimePoint, TrackPosition)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 }, { 0x80000000 | 7, 8 }, { 0x80000000 | 9, 10 },
        }}),
        // Signal 'forceStepDone'
        QtMocHelpers::SignalData<void()>(11, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'loopChanged'
        QtMocHelpers::SignalData<void(PlaybackEngineObjectID, TrackPosition, int)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 }, { 0x80000000 | 9, 13 }, { QMetaType::Int, 14 },
        }}),
        // Slot 'onFinalFrameReceived'
        QtMocHelpers::SlotData<void(PlaybackEngineObjectID)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 16 },
        }}),
        // Slot 'render'
        QtMocHelpers::SlotData<void(Frame)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<Renderer, qt_meta_tag_ZN7QFFmpeg8RendererE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QFFmpeg::Renderer::staticMetaObject = { {
    QMetaObject::SuperData::link<PlaybackEngineObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7QFFmpeg8RendererE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7QFFmpeg8RendererE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN7QFFmpeg8RendererE_t>.metaTypes,
    nullptr
} };

void QFFmpeg::Renderer::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<Renderer *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->frameProcessed((*reinterpret_cast<std::add_pointer_t<Frame>>(_a[1]))); break;
        case 1: _t->synchronized((*reinterpret_cast<std::add_pointer_t<PlaybackEngineObjectID>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<TimePoint>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<TrackPosition>>(_a[3]))); break;
        case 2: _t->forceStepDone(); break;
        case 3: _t->loopChanged((*reinterpret_cast<std::add_pointer_t<PlaybackEngineObjectID>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<TrackPosition>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3]))); break;
        case 4: _t->onFinalFrameReceived((*reinterpret_cast<std::add_pointer_t<PlaybackEngineObjectID>>(_a[1]))); break;
        case 5: _t->render((*reinterpret_cast<std::add_pointer_t<Frame>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (Renderer::*)(Frame )>(_a, &Renderer::frameProcessed, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (Renderer::*)(PlaybackEngineObjectID , TimePoint , TrackPosition )>(_a, &Renderer::synchronized, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (Renderer::*)()>(_a, &Renderer::forceStepDone, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (Renderer::*)(PlaybackEngineObjectID , TrackPosition , int )>(_a, &Renderer::loopChanged, 3))
            return;
    }
}

const QMetaObject *QFFmpeg::Renderer::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QFFmpeg::Renderer::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7QFFmpeg8RendererE_t>.strings))
        return static_cast<void*>(this);
    return PlaybackEngineObject::qt_metacast(_clname);
}

int QFFmpeg::Renderer::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = PlaybackEngineObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 6)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 6;
    }
    return _id;
}

// SIGNAL 0
void QFFmpeg::Renderer::frameProcessed(Frame _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void QFFmpeg::Renderer::synchronized(PlaybackEngineObjectID _t1, TimePoint _t2, TrackPosition _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2, _t3);
}

// SIGNAL 2
void QFFmpeg::Renderer::forceStepDone()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void QFFmpeg::Renderer::loopChanged(PlaybackEngineObjectID _t1, TrackPosition _t2, int _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1, _t2, _t3);
}
QT_WARNING_POP
